#pragma once

#include <utility>
#include <functional>
#include <string>
#include <stdexcept>

#include "common.h"
#include <rngxx.hpp>

namespace rng::init
{
	using ctor_func = std::function<Random* (const void*)>;
	struct DCLookupFailed final : std::exception{ inline DCLookupFailed(const std::string_view& n) : std::exception(), name(std::move(n)){} std::string name; };

	const ctor_func& apply_raw(std::string_view name, ctor_func fn) noexcept;

	template<typename T, typename _Ptr = void>
	inline auto apply(std::string_view name, auto&& lam) noexcept
	{
		return apply_raw(name, [lam](const void* seed) -> Random* {
			return static_cast<Random*>(lam(reinterpret_cast<const _Ptr*>(seed)));
		});
	}

	ctor_func& get(std::string_view name);
}

// To add to dynamic CTOR map
#define RNGXX_APPLY_CTOR(ns, T, TSeed) __attribute__((used))\
const static auto& RNGXX_DCTOR_NAME(T) = rng::init::apply<ns::T, TSeed>(#T, [](const TSeed* ptr) { return new ns::T(*ptr); })

//const extern init::ctor_func& RNGXX_DCTOR_NAME(sm64);
