#include <stdio.h>
#include <stdlib.h>

#include <time.h>

#define _RNGXX_INLINE_ONLY
#include <rngxx.h>


static int next(rng_t* rng, const int* min, const int* max)
{
	int output;
	if(rng_next_tyb(rng, &output, RNG_TY_INT32, RNG_TYQ_SIGNED, min, max)) return output;
	
	fprintf(stderr, "next<int>() failed\n");
	exit(-1);
}

#define TREF(x) ( (const __typeof(x)[]){ (x) } )

const struct { const char* const name; const enum rng_kind static_kind; } DEFAULT_ALG = { "crand", RNG_KIND_CRAND };

int main(int argc, char** argv)
{
	(void)argc;
	const char* alg_name = argv[1] ?: DEFAULT_ALG.name;
	const enum rng_kind* alg_dev = ((int)DEFAULT_ALG.static_kind) == -1 ? NULL : &DEFAULT_ALG.static_kind;

	printf("constructing for %s...\n", alg_name);
	rng_dyn_ctor_ref rcref = rng_ctor_ref(alg_name);
	rng_ctor_fn rctor = rng_ctor(alg_name);

	rng_t* engine = NULL;
	if(rctor) {
		printf("constructing direct\n");
		engine = rctor(TREF((u64)time(NULL)));
	} else {
		printf("constructing through ref\n");
		engine = rng_ctor_call(rcref, TREF((u64)time(NULL)));
	}
	if(!engine) {
		printf("fallback to `rng_new_named()`\n");
		engine = rng_new_named(alg_name, TREF((u64)time(NULL))); //rng_new(rng_kind_crand, (const u64[]){ time(null) });
	}
	if(!engine && alg_dev) {
		printf("fallback to static init of default arg (%d)\n", *alg_dev);
		engine = rng_new(*alg_dev, (const u64[]){ time(NULL) });
	}
	if(!engine) {
		fprintf(stderr, "failed to create engine\n");
		return 1;
	}
	printf("%d %d %d\n", next(engine, NULL, TREF(100)), next(engine, TREF(10), TREF(20)), next(engine, NULL, NULL));	
	printf("%ld\n", rng_next_i64(engine, NULL, NULL));
	printf("%lf\n", rng_next_f64(engine, TREF(1.0), TREF(100.0)));
	printf("%d\n", rng_next_bool(engine));

	
	rng_bool_t* array = rng_nextam_bool(engine, 100);
	if(!array) {
		fprintf(stderr, "failed to allocate for array\n");
		return 1;
	}
	for(int i=0;i<100;i++)
		printf("%d ", array[i]);
	printf("\n"); free(array);

	int ints[100];
	if(!rng_nexta_i32(engine, ints, 100, NULL, TREF(100))) {
		fprintf(stderr, "failed to compute int array\n");
		return 1;
	}
	
	for(size_t i=0;i<sizeof(ints)/sizeof(ints[0]);i++)
		printf("%d ", ints[i]);
	printf("\n");

	rng_free(engine);

	return 0;
}
